@@ -1,5 +1,5 @@
-#! /usr/local/bin/ruby -w
-require 'RMagick'
+#!/usr/bin/env ruby -w
+require 'rmagick'
 
 imgl = Magick::ImageList.new
 imgl.new_image(390, 360, Magick::HatchFill.new('white','lightcyan2'))
@@ -36,8 +36,7 @@ gc.text(329,330, "'320,330'")
 gc.text(329,180, "'320,180'")
 
 gc.draw(imgl)
-imgl.border!(1,1, "lightcyan2")
+imgl.border!(1,1, 'lightcyan2')
 
 imgl.write('cbezier4.gif')
 exit(0)
-
