@@ -497,24 +497,26 @@ Draw_interword_spacing_eq(VALUE self, VALUE spacing)
 static VALUE
 image_to_str(Image *image)
 {
-    volatile VALUE dimg = Qnil;
+    VALUE dimg = Qnil;
     unsigned char *blob;
     size_t length;
     Info *info;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     if (image)
     {
         info = CloneImageInfo(NULL);
-        GetExceptionInfo(&exception);
-        blob = ImageToBlob(info, image, &length, &exception);
+        exception = AcquireExceptionInfo();
+        blob = ImageToBlob(info, image, &length, exception);
         DestroyImageInfo(info);
         CHECK_EXCEPTION();
-        DestroyExceptionInfo(&exception);
+        DestroyExceptionInfo(exception);
         dimg = rb_str_new((char *)blob, (long)length);
         magick_free((void*)blob);
     }
 
+    RB_GC_GUARD(dimg);
+
     return dimg;
 }
 
@@ -536,16 +538,16 @@ Image *str_to_image(VALUE str)
 {
     Image *image = NULL;
     Info *info;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     if (str != Qnil)
     {
         info = CloneImageInfo(NULL);
-        GetExceptionInfo(&exception);
-        image = BlobToImage(info, RSTRING_PTR(str), RSTRING_LEN(str), &exception);
+        exception = AcquireExceptionInfo();
+        image = BlobToImage(info, RSTRING_PTR(str), RSTRING_LEN(str), exception);
         DestroyImageInfo(info);
         CHECK_EXCEPTION();
-        DestroyExceptionInfo(&exception);
+        DestroyExceptionInfo(exception);
     }
 
     return image;
@@ -664,7 +666,7 @@ Draw_marshal_load(VALUE self, VALUE ddraw)
 {
     Draw *draw;
     Pixel *pixel;
-    volatile VALUE val;
+    VALUE val;
 
     Data_Get_Struct(self, Draw, draw);
 
@@ -725,6 +727,8 @@ Draw_marshal_load(VALUE self, VALUE ddraw)
 
     draw->primitives = rb_hash_aref(ddraw, CSTR2SYM("primitives"));
 
+    RB_GC_GUARD(val);
+
     return self;
 }
 
@@ -1051,7 +1055,7 @@ VALUE Draw_annotate(
 VALUE
 Draw_clone(VALUE self)
 {
-    volatile VALUE clone;
+    VALUE clone;
 
     clone = Draw_dup(self);
     if (OBJ_FROZEN(self))
@@ -1059,6 +1063,8 @@ Draw_clone(VALUE self)
         OBJ_FREEZE(clone);
     }
 
+    RB_GC_GUARD(clone);
+
     return clone;
 }
 
@@ -1087,7 +1093,7 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     const char *op = "Over";
     double x, y, width, height;
     CompositeOperator cop = OverCompositeOp;
-    volatile VALUE image;
+    VALUE image;
     Image *comp_img;
     struct TmpFile_Name *tmpfile_name;
     char name[MaxTextExtent];
@@ -1339,6 +1345,8 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     // Send "primitive" to self.
     (void) rb_funcall(self, rb_intern("primitive"), 1, rb_str_new2(primitive));
 
+    RB_GC_GUARD(image);
+
     return self;
 }
 
@@ -1399,7 +1407,7 @@ VALUE
 Draw_dup(VALUE self)
 {
     Draw *draw;
-    volatile VALUE dup;
+    VALUE dup;
 
     draw = ALLOC(Draw);
     memset(draw, 0, sizeof(Draw));
@@ -1408,6 +1416,9 @@ Draw_dup(VALUE self)
     {
         (void)rb_obj_taint(dup);
     }
+
+    RB_GC_GUARD(dup);
+
     return rb_funcall(dup, rm_ID_initialize_copy, 1, self);
 }
 
@@ -1513,7 +1524,7 @@ VALUE
 Draw_initialize(VALUE self)
 {
     Draw *draw, *draw_options;
-    volatile VALUE options;
+    VALUE options;
 
     Data_Get_Struct(self, Draw, draw);
 
@@ -1522,6 +1533,8 @@ Draw_initialize(VALUE self)
     draw->info = draw_options->info;
     draw_options->info = NULL;
 
+    RB_GC_GUARD(options);
+
     return self;
 }
 
@@ -1560,12 +1573,14 @@ Draw_inspect(VALUE self)
 VALUE Draw_alloc(VALUE class)
 {
     Draw *draw;
-    volatile VALUE draw_obj;
+    VALUE draw_obj;
 
     draw = ALLOC(Draw);
     memset(draw, 0, sizeof(Draw));
     draw_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw);
 
+    RB_GC_GUARD(draw_obj);
+
     return draw_obj;
 }
 
@@ -1685,12 +1700,14 @@ VALUE
 DrawOptions_alloc(VALUE class)
 {
     Draw *draw_options;
-    volatile VALUE draw_options_obj;
+    VALUE draw_options_obj;
 
     draw_options = ALLOC(Draw);
     memset(draw_options, 0, sizeof(Draw));
     draw_options_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw_options);
 
+    RB_GC_GUARD(draw_options_obj);
+
     return draw_options_obj;
 }
 
@@ -1745,7 +1762,7 @@ DrawOptions_initialize(VALUE self)
 VALUE
 PolaroidOptions_alloc(VALUE class)
 {
-    volatile VALUE polaroid_obj;
+    VALUE polaroid_obj;
     ImageInfo *image_info;
     Draw *draw;
 
@@ -1759,6 +1776,8 @@ PolaroidOptions_alloc(VALUE class)
 
     polaroid_obj = Data_Wrap_Struct(class, NULL, destroy_Draw, draw);
 
+    RB_GC_GUARD(polaroid_obj);
+
     return polaroid_obj;
 }
 
@@ -1776,15 +1795,15 @@ VALUE
 PolaroidOptions_initialize(VALUE self)
 {
     Draw *draw;
-    ExceptionInfo exception;
+    ExceptionInfo *exception;
 
     // Default shadow color
     Data_Get_Struct(self, Draw, draw);
 
-    GetExceptionInfo(&exception);
-    (void) QueryColorDatabase("gray75", &draw->shadow_color, &exception);
+    exception = AcquireExceptionInfo();
+    (void) QueryColorDatabase("gray75", &draw->shadow_color, exception);
     CHECK_EXCEPTION()
-    (void) QueryColorDatabase("#dfdfdf", &draw->info->border_color, &exception);
+    (void) QueryColorDatabase("#dfdfdf", &draw->info->border_color, exception);
 
     if (rb_block_given_p())
     {
@@ -1863,7 +1882,7 @@ static VALUE
 get_dummy_tm_img(VALUE klass)
 {
 #define DUMMY_IMG_CLASS_VAR "@@_dummy_img_"
-    volatile VALUE dummy_img = 0;
+    VALUE dummy_img = 0;
     Info *info;
     Image *image;
 
@@ -1887,6 +1906,8 @@ get_dummy_tm_img(VALUE klass)
     }
     dummy_img = rb_cv_get(klass, DUMMY_IMG_CLASS_VAR);
 
+    RB_GC_GUARD(dummy_img);
+
     return dummy_img;
 }
 
@@ -1918,7 +1939,7 @@ get_type_metrics(
  #define ATTRS_L ((int)(sizeof(attrs)-1))
     Image *image;
     Draw *draw;
-    volatile VALUE t;
+    VALUE t;
     TypeMetric metrics;
     char *text = NULL;
     long text_l;
@@ -1930,21 +1951,28 @@ get_type_metrics(
         case 1:                   // use default image
             text = rm_str2cstr(argv[0], &text_l);
 
-            for (x = 0; x < text_l; x++)
+            for (x = 0; x < text_l-1; x++)
             {
                 // Ensure text string doesn't refer to image attributes.
-                if (text[x] == '%' && x < text_l-1)
+                if (text[x] == '%')
                 {
                     int y;
                     char spec = text[x+1];
 
-                    for (y = 0; y < ATTRS_L; y++)
+                    if (spec == '%')
                     {
-                        if (spec == attrs[y])
+                        x++;
+                    }
+                    else
+                    {
+                        for (y = 0; y < ATTRS_L; y++)
                         {
-                            rb_raise(rb_eArgError,
-                                     "text string contains image attribute reference `%%%c'",
-                                     spec);
+                            if (spec == attrs[y])
+                            {
+                                rb_raise(rb_eArgError,
+                                         "text string contains image attribute reference `%%%c'",
+                                         spec);
+                            }
                         }
                     }
                 }
@@ -1987,5 +2015,8 @@ get_type_metrics(
         rb_raise(rb_eRuntimeError, "Can't measure text. Are the fonts installed? "
                  "Is the FreeType library installed?");
     }
+
+    RB_GC_GUARD(t);
+
     return Import_TypeMetric(&metrics);
 }
